package gmodel

import (
	"context"
	"time"

	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

func (a *FsAddressModel) GetOne(ctx context.Context, addressId int64, userId int64) (resp *FsAddress, err error) {
	err = a.db.WithContext(ctx).Model(&FsAddress{}).Where("`address_id` = ? and `user_id` = ? and `status` = ? ", addressId, userId, 1).Take(&resp).Error
	return resp, err
}

func (a *FsAddressModel) GetUserAllAddress(ctx context.Context, userId int64) (resp []FsAddress, err error) {
	err = a.db.WithContext(ctx).Model(&FsAddress{}).Where("`user_id` = ? and `status` = ?", userId, 1).Order("`ltime` DESC").Find(&resp).Error
	if err != nil {
		return nil, err
	}
	return
}

func (a *FsAddressModel) CreateOne(ctx context.Context, address *FsAddress) (result *FsAddress, err error) {

	err = a.db.WithContext(ctx).Model(&FsAddress{}).Transaction(func(tx *gorm.DB) error {
		now := time.Now().UTC()
		result = &FsAddress{
			UserId:      address.UserId,
			AddressName: address.AddressName,
			FirstName:   address.FirstName,
			LastName:    address.LastName,
			Mobile:      address.Mobile,
			Street:      address.Street,
			Suite:       address.Suite,
			City:        address.City,
			State:       address.State,
			Country:     address.Country,
			ZipCode:     address.ZipCode,
			Status:      address.Status,
			IsDefault:   address.IsDefault,
			Ctime:       &now,
			Utime:       &now,
			Ltime:       &now,
		}

		// lastOne := &FsAddress{}
		// err = tx.Where("user_id = ?", lastOne.UserId).Order("ltime ASC").Take(&lastOne).Error
		// if err == gorm.ErrRecordNotFound {
		// 	result.Ltime = &now
		// 	return tx.Model(&FsAddress{}).Create(result).Error
		// }
		// if err != nil {
		// 	return err
		// }

		// // 根据lastOne处理时间

		// ltime := (*lastOne.Ltime).Add(-time.Second)
		// result.Ltime = &ltime
		return tx.Model(&FsAddress{}).Create(result).Error
	})

	if err != nil {
		return nil, err
	}

	return result, nil
}

func (a *FsAddressModel) UpdateAddAddress(ctx context.Context, address *FsAddress) (err error) {
	err = a.db.WithContext(ctx).Transaction(func(tx *gorm.DB) error {
		if *address.IsDefault > 0 {
			err = tx.Model(&FsAddress{}).Where("user_id = ? and is_default = 1 ", address.UserId).Update("is_default", 0).Error
			if err != nil {
				return err
			}
		}
		return tx.Model(&FsAddress{}).Where("id = ? and user_id = ?", address.AddressId, address.UserId).Omit("id", "user_id").Updates(address).Error
	})
	return err
}

func (a *FsAddressModel) SettingUserDefaultAddress(ctx context.Context, userId int64, addressId int64) (resp []FsAddress, err error) {

	a.db.WithContext(ctx).Model(&FsAddress{}).Transaction(func(tx *gorm.DB) error {

		now := time.Now().UTC()

		err = tx.Where(" `user_id` = ? and `status` = ? and `address_id = ?` ", userId, 1, addressId).
			UpdateColumn("ltime", now.Add(time.Hour*24*365*290)).
			UpdateColumn("utime", now).Error
		if err != nil {
			return err
		}

		err = tx.Where(" `user_id` = ? and `status` = ? and `address_id != ? and `ltime` > ?` ", userId, 1, addressId, now.Add(time.Hour*24*365)).
			UpdateColumn("ltime", now).Error
		if err != nil {
			logx.Error(err)
		}

		return nil
	})

	return
}
